package ca.canadapost.cpcdp.rating;

import java.math.BigDecimal;
import java.util.Map;

import ca.canadapost.cpcdp.rating.stub.GetRatesRequest;
import ca.canadapost.cpcdp.rating.stub.GetRatesResponse;
import ca.canadapost.cpcdp.rating.stub.RatingPortType;
import ca.canadapost.cpcdp.rating.stub.RatingService;
import ca.canadapost.cpcdp.rating.stub.Messages;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.rating.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

/**
 * Sample code for the GetRates Canada Post service.
 * <p>
 * The GetRates service returns a list of shipping services, prices and transit times 
 * for a given item to be shipped. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class GetRatesClient {
    	
    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
           	// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Rating_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            RatingService service = new RatingService();
            RatingPortType port = service.getRatingPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/rating/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            GetRatesRequest.MailingScenario.ParcelCharacteristics parcelCharacteristics = new GetRatesRequest.MailingScenario.ParcelCharacteristics();
            parcelCharacteristics.setWeight(new BigDecimal(1));

            GetRatesRequest.MailingScenario.Destination.Domestic domestic = new GetRatesRequest.MailingScenario.Destination.Domestic();
            domestic.setPostalCode("J0E1X0");

            GetRatesRequest.MailingScenario.Destination destination = new GetRatesRequest.MailingScenario.Destination();
            destination.setDomestic(domestic);

            GetRatesRequest.MailingScenario mailingScenario = new GetRatesRequest.MailingScenario();
            mailingScenario.setCustomerNumber(mailedBy);
            mailingScenario.setParcelCharacteristics(parcelCharacteristics);
            mailingScenario.setOriginPostalCode("K2B8J6");
            mailingScenario.setDestination(destination);

            GetRatesRequest request = new GetRatesRequest();
            request.setLocale("FR");
            request.setMailingScenario(mailingScenario);

            //execute request
            GetRatesResponse response = port.getRates(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getPriceQuotes() != null){
                    for(GetRatesResponse.PriceQuotes.PriceQuote priceQuote : response.getPriceQuotes().getPriceQuote()){
                        System.out.println("Service Name: " + priceQuote.getServiceName());
                        System.out.println("Price: $" + priceQuote.getPriceDetails().getDue());
                        System.out.println();
                    }
                }
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
